/**
Author:  Luis Gerardo de la Fraga
Scubes is a program to visualize vox-solids
Copyright (C) 2001 Luis Gerardo de la Fraga

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Author's e-mail: fraga@cs.cinvestav.mx
Address: 
Seccin de Computacin. Dpto. de Ingeniera Elctrica
CINVESTAV-IPN                       
Av. Instituto Politcnico Nacional No. 2508
07300 Mxico, D.F. 
Mxico
**/

/** 
	A program in Qt
	that use OpenGL's drawing primitives
	for visualize the surface of a volume, 
	formed it with voxels

	File: scubes.cpp

	Fraga, March 26, 2001

	Name changed to "scubes.cpp", March 10, 2001

**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <qgl.h>
#include <qpainter.h>
#include <qmessagebox.h> 
#include <qpixmap.h> 
#include <qimage.h> 
#include "quadric.h"

Quadric::Quadric( const char *fname, QWidget *parent, const char *name )
		: QGLWidget( parent, name )
{
	diffuseMaterial[0] = 250.0/255.0;
	diffuseMaterial[1] = 250.0/255.0;
	diffuseMaterial[2] = 250.0/255.0;
	diffuseMaterial[3] = 1.0;
	lightfixedto = 0; // 0 = to viewer,  1 = to object
	scale = 1.0;

	theta = phi = 0.0;
	xrot = yrot = 0;
}


void Quadric::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );

	theta = (float)xmouse*360.f/(float)windowWidth - 180.f;
	phi = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

//
// This method puts the initial coordinates each time window is resized
//
void Quadric::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	float d = 1.0;
	if( w <= h )
		glOrtho (-d, d, -d/aspect, d/aspect,-d, d);
	else
		glOrtho (-aspect*d, aspect*d, -d, d,-d, d);

}

QSizePolicy Quadric::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Quadric::paintGL(void)
{
	static float position0[] = {-1.0, 1.0, 1.0, 1.0};



	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */

	if ( !lightfixedto ) {
		glPushMatrix( );
		/* viewing transformation  */
		glScalef ( scale, scale, scale);	/* modeling transformation */ 
		glLightfv(GL_LIGHT0, GL_POSITION, position0);
		glPopMatrix( );
	}

	glRotatef (theta, 0.0, 0.0, 1.0);

	glPushMatrix( );

	double yprimex = sin( M_PI*theta/180 );
	double yprimey = cos( M_PI*theta/180 );

	// sincos( M_PI*theta/180, &yprimex, &yprimey );

	glRotatef (phi, yprimex, yprimey, 0.0);

	if ( lightfixedto ) {
		glPushMatrix( );
		/* viewing transformation  */
		glScalef ( scale, scale, scale);	/* modeling transformation */ 
		glLightfv(GL_LIGHT0, GL_POSITION, position0);
		glPopMatrix( );
	}

	glCallList( startList );

	glPopMatrix( );

	swapBuffers();
}

void Quadric::initializeGL( ) 
{
	glClearColor (0.85, 0.85, 0.85, 0.0);	// Background to a grey tone
	// glClearColor ( 1.0, 1.0, 1.0, 0.0);	// Background to white

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	float d = 1.0;
	glOrtho(-d, d, -d, d, -d, d);


	glEnable (GL_DEPTH_TEST);

	glEnable( GL_LIGHTING );
	glShadeModel( GL_FLAT );

	InitMaterials();

	startList = glGenLists( 1 );
	obj0 = gluNewQuadric( );
	gluQuadricCallback( obj0, GLU_ERROR, (void (*)(void))errorCallBack );

	gluQuadricNormals( obj0, GLU_FLAT );
	glNewList( startList, GL_COMPILE );
		gluSphere( obj0, 0.85, 25, 15);
	glEndList();
	
}

void errorCallBack( GLenum errorCode )
{
	const GLubyte *errors;

	errors = gluErrorString( errorCode );

	fprintf( stderr, "Quadric error: %s\n", errors );
	// QMessageBox::critical( this, "Error in OpenGL", 
	// 		QString( (const char *)errors ) );

	// exit( ): // ?	
}

void Quadric::InitMaterials(void)
{
	static float ambient[] = {0.4, 0.4, 0.4, 1.0};
	static float diffuse[] = {1.0, 1.0, 1.0, 1.0};
	static float specular[] = {0.0, 0.0, 0.0, 1.0};



	glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
	// glLightfv(GL_LIGHT0, GL_SPECULAR, position0);
	glEnable(GL_LIGHT0);

	// static float mat_ambient[] = {0.2, 0.2, 0.2, 1.0};
	glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, diffuseMaterial );
	// glMaterialfv( GL_FRONT, GL_AMBIENT, mat_ambient );


	static float lmodel_ambient[] = {0.1, 0.1, 0.1, 1.0};
	glLightModelfv( GL_LIGHT_MODEL_AMBIENT, lmodel_ambient );
}


